from ..object_compare_tracking import ObjectComparisonTracking
from enstools.feature.identification._proto_gen import template_pb2


# TODO need template for non-comparison trackings?
class TrackingCompareTemplate(ObjectComparisonTracking):
    # Template for simple object-comparision tracking: Objects of consecutive timestamps are considered the same
    # if correspond() returns True.
    # This can be used if the tracking can simply be done by this pairwise comparision. More complex algorithms, which
    # rely on a broader state or which need information from timesteps before, need a more general approach than this.

    def __init__(self, **kwargs):
        # called from example_template.py, to set parameters.
        pass

    def correspond(self, time1, obj1, time2, obj2):
        # Compare obj1 and obj2, based on their .proto definition
        # Each object has by default an ID, it can have a positional representation (volume, boundary), and it can have
        # the descriptions set in the identification setup, represented by the .proto file (e.g. template_pb2).

        # For example, if we want to track objects from the identificaton template, we can compare my_classification,
        # size or their centroids.

        # Assume this is used with IdentificationTemplate
        # get ID of this object, unique for given timestep
        o1_id = obj1.id
        o2_id = obj2.id

        # access the properties set earlier
        properties1 = obj1.properties
        properties2 = obj2.properties

        if properties1.my_classification > properties2.my_classification:
            # some random heuristic, if this is True, obj1 at timestep t and obj2 at timestep t+1 are the same
            return True

        return False
